"use strict";

exports.__esModule = true;
exports.ComboboxOption = exports.ComboboxList = exports.ComboboxPopup = exports.ComboboxInput = exports.Combobox = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _on, _on2, _on3, _on4, _states; /* eslint-disable jsx-a11y/role-has-required-aria-props */
/* eslint-disable jsx-a11y/aria-proptypes */
/* eslint-disable jsx-a11y/role-supports-aria-props */
/* eslint-disable default-case */

////////////////////////////////////////////////////////////////////////////////
// Welcome to @reach/combobox! State transitions are managed by a state chart,
// state mutations are managed by a reducer. Please enjoy the read here, I
// figured out a few new tricks with context and refs I think you might love or
// hate 😂

exports.useComboboxContextAndYouWillBeFired = useComboboxContextAndYouWillBeFired;
exports.ComboboxOptionText = ComboboxOptionText;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _utils = require("@reach/utils");

var _highlightWordsCore = require("highlight-words-core");

var _escapeRegexp = require("escape-regexp");

var _escapeRegexp2 = _interopRequireDefault(_escapeRegexp);

var _autoId = require("@reach/auto-id");

var _popover = require("@reach/popover");

var _popover2 = _interopRequireDefault(_popover);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

////////////////////////////////////////////////////////////////////////////////
// States

// Nothing going on, waiting for the user to type or use the arrow keys
var IDLE = "IDLE";

// The component is suggesting options as the user types
var SUGGESTING = "SUGGESTING";

// The user is using the keyboard to navigate the list, not typing
var NAVIGATING = "NAVIGATING";

// The user is interacting with arbitrary elements inside the popup that
// are not ComboboxInputs
var INTERACTING = "INTERACTING";

////////////////////////////////////////////////////////////////////////////////
// Actions:

// User cleared the value w/ backspace, but input still has focus
var CLEAR = "CLEAR";

// User is typing
var CHANGE = "CHANGE";

// User is navigating w/ the keyboard
var NAVIGATE = "NAVIGATE";

// User can be navigating with keyboard and then
// click instead, we want the value from the click,
// not the current nav item
var SELECT_WITH_KEYBOARD = "SELECT_WITH_KEYBOARD";
var SELECT_WITH_CLICK = "SELECT_WITH_CLICK";

// Pretty self-explanatory, user can hit escape or
// blur to close the popover
var ESCAPE = "ESCAPE";
var BLUR = "BLUR";

// The user left the input to interact with arbitrary elements inside the
// popup
var INTERACT = "INTERACT";

////////////////////////////////////////////////////////////////////////////////
var stateChart = {
  initial: IDLE,
  states: (_states = {}, _states[IDLE] = {
    on: (_on = {}, _on[BLUR] = IDLE, _on[CLEAR] = IDLE, _on[CHANGE] = SUGGESTING, _on[NAVIGATE] = NAVIGATING, _on)
  }, _states[SUGGESTING] = {
    on: (_on2 = {}, _on2[CHANGE] = SUGGESTING, _on2[NAVIGATE] = NAVIGATING, _on2[CLEAR] = IDLE, _on2[ESCAPE] = IDLE, _on2[BLUR] = IDLE, _on2[SELECT_WITH_CLICK] = IDLE, _on2[INTERACT] = INTERACTING, _on2)
  }, _states[NAVIGATING] = {
    on: (_on3 = {}, _on3[CHANGE] = SUGGESTING, _on3[CLEAR] = IDLE, _on3[BLUR] = IDLE, _on3[ESCAPE] = IDLE, _on3[NAVIGATE] = NAVIGATING, _on3[SELECT_WITH_KEYBOARD] = IDLE, _on3[SELECT_WITH_CLICK] = IDLE, _on3[INTERACT] = INTERACTING, _on3)
  }, _states[INTERACTING] = {
    on: (_on4 = {}, _on4[CHANGE] = SUGGESTING, _on4[BLUR] = IDLE, _on4[ESCAPE] = IDLE, _on4[NAVIGATE] = NAVIGATING, _on4)
  }, _states)
};

function reducer(data, action) {
  var nextState = _extends({}, data, { lastActionType: action.type });
  switch (action.type) {
    case CHANGE:
      return _extends({}, nextState, {
        navigationValue: null,
        value: action.value
      });
    case NAVIGATE:
      return _extends({}, nextState, {
        navigationValue: action.value
      });
    case CLEAR:
      return _extends({}, nextState, {
        value: "",
        navigationValue: null
      });
    case BLUR:
    case ESCAPE:
      return _extends({}, nextState, {
        navigationValue: null
      });
    case SELECT_WITH_CLICK:
      return _extends({}, nextState, {
        value: action.value,
        navigationValue: null
      });
    case SELECT_WITH_KEYBOARD:
      return _extends({}, nextState, {
        value: data.navigationValue,
        navigationValue: null
      });
    case INTERACT:
      return _extends({}, nextState, { navigationValue: null });

    default:
      throw new Error("Unknown action " + action.type);
  }
}

var visibleStates = [SUGGESTING, NAVIGATING, INTERACTING];
var isVisible = function isVisible(state) {
  return visibleStates.includes(state);
};

////////////////////////////////////////////////////////////////////////////////
// Combobox

var Context = (0, _react.createContext)();

function useComboboxContextAndYouWillBeFired() {
  return (0, _react.useContext)(Context);
}

var Combobox = (0, _react.forwardRef)(function Combobox(_ref, ref) {
  var children = _ref.children,
      _ref$as = _ref.as,
      Comp = _ref$as === undefined ? "div" : _ref$as,
      onSelect = _ref.onSelect,
      rest = _objectWithoutProperties(_ref, ["children", "as", "onSelect"]);

  // We store the values of all the ComboboxOptions on this ref. This makes it
  // possible to perform the keyboard navigation from the input on the list. We
  // manipulate this array through context so that we don't have to enforce a
  // parent/child relationship between ComboboxList and ComboboxOption with
  // cloneElement or fall back to DOM traversal. It's a new trick for me and
  // I'm pretty excited about it.
  var optionsRef = (0, _react.useRef)(null);

  // Need this to focus it
  var inputRef = (0, _react.useRef)();

  var popupRef = (0, _react.useRef)();

  // When <ComboboxInput autocomplete={false} /> we don't want cycle back to
  // the user's value while navigating (because it's always the user's value),
  // but we need to know this in useKeyDown which is far away from the prop
  // here, so we do something sneaky and write it to this ref on context so we
  // can use it anywhere else 😛. Another new trick for me and I'm excited
  // about this one too!
  var autocompletePropRef = (0, _react.useRef)();

  var defaultData = {
    // the value the user has typed, we derived this also when the developer is
    // controlling the value of ComboboxInput
    value: "",
    // the value the user has navigated to with the keyboard
    navigationValue: null
  };

  var _useReducerMachine = useReducerMachine(stateChart, reducer, defaultData),
      state = _useReducerMachine[0],
      data = _useReducerMachine[1],
      transition = _useReducerMachine[2];

  useFocusManagement(data.lastActionType, inputRef, data.navigationValue);

  var listboxId = "listbox:" + (0, _autoId.useId)();

  var context = (0, _react.useMemo)(function () {
    return {
      data: data,
      inputRef: inputRef,
      popupRef: popupRef,
      onSelect: onSelect,
      optionsRef: optionsRef,
      state: state,
      transition: transition,
      listboxId: listboxId,
      autocompletePropRef: autocompletePropRef
    };
  }, [data, onSelect, state, transition, listboxId]);

  return _react2.default.createElement(
    Context.Provider,
    { value: context },
    _react2.default.createElement(
      Comp,
      _extends({}, rest, {
        "data-reach-combobox": "",
        ref: ref,
        role: "combobox",
        "aria-haspopup": "listbox",
        "aria-owns": listboxId,
        "aria-expanded": isVisible(state)
      }),
      children
    )
  );
});

exports.Combobox = Combobox;
process.env.NODE_ENV !== "production" ? Combobox.propTypes = { onSelect: _propTypes.func } : void 0;

////////////////////////////////////////////////////////////////////////////////
// ComboboxInput

var ComboboxInput = (0, _react.forwardRef)(function ComboboxInput(_ref2, forwardedRef) {
  var _ref2$as = _ref2.as,
      Comp = _ref2$as === undefined ? "input" : _ref2$as,
      _ref2$selectOnClick = _ref2.selectOnClick,
      selectOnClick = _ref2$selectOnClick === undefined ? false : _ref2$selectOnClick,
      _ref2$autocomplete = _ref2.autocomplete,
      autocomplete = _ref2$autocomplete === undefined ? true : _ref2$autocomplete,
      onClick = _ref2.onClick,
      onChange = _ref2.onChange,
      onKeyDown = _ref2.onKeyDown,
      onBlur = _ref2.onBlur,
      onFocus = _ref2.onFocus,
      controlledValue = _ref2.value,
      props = _objectWithoutProperties(_ref2, ["as", "selectOnClick", "autocomplete", "onClick", "onChange", "onKeyDown", "onBlur", "onFocus", "value"]);

  var _useContext = (0, _react.useContext)(Context),
      _useContext$data = _useContext.data,
      navigationValue = _useContext$data.navigationValue,
      value = _useContext$data.value,
      inputRef = _useContext.inputRef,
      state = _useContext.state,
      transition = _useContext.transition,
      listboxId = _useContext.listboxId,
      autocompletePropRef = _useContext.autocompletePropRef;

  // Because we close the List on blur, we need to track if the blur is
  // caused by clicking inside the list, and if so, don't close the List.


  var selectOnClickRef = (0, _react.useRef)(false);

  var handleKeyDown = useKeyDown();

  var handleBlur = useBlur();

  var isControlled = controlledValue != null;

  (0, _react.useLayoutEffect)(function () {
    autocompletePropRef.current = autocomplete;
  });

  var handleValueChange = function handleValueChange(value) {
    if (value.trim() === "") {
      transition(CLEAR);
    } else {
      transition(CHANGE, { value: value });
    }
  };

  // If they are controlling the value we still need to do our transitions so
  // we have this derived state to emulate onChange of the input as we receive
  // new `value`s ...
  if (isControlled && controlledValue !== value) {
    handleValueChange(controlledValue);
  }

  // ... and we don't trigger handleValueChange as the user types, we just let
  // the developer control it with the normal input onChange prop
  var handleChange = function handleChange(event) {
    if (!isControlled) {
      handleValueChange(event.target.value);
    }
  };

  var handleFocus = function handleFocus() {
    if (selectOnClick) {
      selectOnClickRef.current = true;
    }
  };

  var handleClick = function handleClick() {
    if (selectOnClickRef.current) {
      selectOnClickRef.current = false;
      inputRef.current.select();
    }
  };

  var inputValue = autocomplete && state === NAVIGATING ? // When idle, we don't have a navigationValue on ArrowUp/Down
  navigationValue || controlledValue || value : controlledValue || value;

  return _react2.default.createElement(Comp, _extends({}, props, {
    "data-reach-combobox-input": "",
    ref: function ref(node) {
      (0, _utils.assignRef)(inputRef, node);
      (0, _utils.assignRef)(forwardedRef, node);
    },
    value: inputValue,
    onClick: (0, _utils.wrapEvent)(onClick, handleClick),
    onBlur: (0, _utils.wrapEvent)(onBlur, handleBlur),
    onFocus: (0, _utils.wrapEvent)(onFocus, handleFocus),
    onChange: (0, _utils.wrapEvent)(onChange, handleChange),
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, handleKeyDown),
    id: listboxId,
    "aria-autocomplete": "both",
    "aria-activedescendant": navigationValue ? makeHash(navigationValue) : undefined
  }));
});

////////////////////////////////////////////////////////////////////////////////
// ComboboxPopup

exports.ComboboxInput = ComboboxInput;
var ComboboxPopup = (0, _react.forwardRef)(function ComboboxPopup(_ref3, forwardedRef) {
  var onKeyDown = _ref3.onKeyDown,
      onBlur = _ref3.onBlur,
      props = _objectWithoutProperties(_ref3, ["onKeyDown", "onBlur"]);

  var _useContext2 = (0, _react.useContext)(Context),
      state = _useContext2.state,
      popupRef = _useContext2.popupRef,
      inputRef = _useContext2.inputRef;

  var handleKeyDown = useKeyDown();
  var handleBlur = useBlur();

  // Instead of conditionally rendering the popover we use the `hidden` prop
  // because we don't want to unmount on close (from escape or onSelect).  If
  // we unmounted, then we'd lose the optionsRef and the user wouldn't be able
  // to use the arrow keys to pop the list back open. However, the developer
  // can conditionally render the ComboboxPopup if they do want to cause
  // mount/unmount based on the app's own data (like results.length or
  // whatever).
  var hidden = !isVisible(state);

  return _react2.default.createElement(_popover2.default, _extends({}, props, {
    "data-reach-combobox-popup": "",
    targetRef: inputRef,
    position: _popover.positionMatchWidth,
    ref: function ref(node) {
      (0, _utils.assignRef)(popupRef, node);
      (0, _utils.assignRef)(forwardedRef, node);
    },
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, handleKeyDown),
    onBlur: (0, _utils.wrapEvent)(onBlur, handleBlur),
    hidden: hidden
    // Allow the user to click inside the popover without causing it to blur
    // and close.
    , tabIndex: "-1"
  }));
});

////////////////////////////////////////////////////////////////////////////////
// ComboboxList

exports.ComboboxPopup = ComboboxPopup;
var ComboboxList = (0, _react.forwardRef)(function ComboboxList(_ref4, ref) {
  var _ref4$as = _ref4.as,
      Comp = _ref4$as === undefined ? "ul" : _ref4$as,
      props = _objectWithoutProperties(_ref4, ["as"]);

  var _useContext3 = (0, _react.useContext)(Context),
      optionsRef = _useContext3.optionsRef;

  // WEIRD? Reset the options ref every render so that they are always
  // accurate and ready for keyboard navigation handlers. Using layout
  // effect to schedule this effect before the ComboboxOptions push into
  // the array


  (0, _react.useLayoutEffect)(function () {
    optionsRef.current = [];
    return function () {
      return optionsRef.current = [];
    };
  });

  return _react2.default.createElement(Comp, _extends({}, props, { ref: ref, "data-reach-combobox-list": "", role: "listbox" }));
});

////////////////////////////////////////////////////////////////////////////////
// ComboboxOption

// Allows us to put the option's value on context so that ComboboxOptionText
// can work it's highlight text magic no matter what else is rendered around
// it.
exports.ComboboxList = ComboboxList;
var OptionContext = (0, _react.createContext)();

var ComboboxOption = (0, _react.forwardRef)(function ComboboxOption(_ref5, ref) {
  var children = _ref5.children,
      value = _ref5.value,
      onClick = _ref5.onClick,
      onMouseDown = _ref5.onMouseDown,
      props = _objectWithoutProperties(_ref5, ["children", "value", "onClick", "onMouseDown"]);

  var _useContext4 = (0, _react.useContext)(Context),
      onSelect = _useContext4.onSelect,
      navigationValue = _useContext4.data.navigationValue,
      transition = _useContext4.transition,
      optionsRef = _useContext4.optionsRef;

  (0, _react.useEffect)(function () {
    optionsRef.current.push(value);
  });

  var isActive = navigationValue === value;

  var handleClick = function handleClick() {
    onSelect && onSelect(value);
    transition(SELECT_WITH_CLICK, { value: value });
  };

  return _react2.default.createElement(
    OptionContext.Provider,
    { value: value },
    _react2.default.createElement("li", _extends({}, props, {
      "data-reach-combobox-option": "",
      ref: ref,
      id: makeHash(value),
      role: "option",
      "aria-selected": isActive
      // without this the menu will close from `onBlur`, but with it the
      // element can be `document.activeElement` and then our focus checks in
      // onBlur will work as intended
      , tabIndex: "-1",
      onClick: (0, _utils.wrapEvent)(onClick, handleClick),
      children: children || _react2.default.createElement(ComboboxOptionText, null)
    }))
  );
});

////////////////////////////////////////////////////////////////////////////////
// ComboboxOptionText

// We don't forwardRef or spread props because we render multiple spans or null,
// should be fine 🤙
exports.ComboboxOption = ComboboxOption;
function ComboboxOptionText() {
  var value = (0, _react.useContext)(OptionContext);

  var _useContext5 = (0, _react.useContext)(Context),
      contextValue = _useContext5.data.value;

  var searchWords = (0, _escapeRegexp2.default)(contextValue).split(/\s+/);
  var textToHighlight = value;
  var results = (0, _react.useMemo)(function () {
    return (0, _highlightWordsCore.findAll)({ searchWords: searchWords, textToHighlight: textToHighlight });
  }, [searchWords, textToHighlight]);

  return results.length ? results.map(function (result, index) {
    var str = value.slice(result.start, result.end);
    return _react2.default.createElement(
      "span",
      {
        key: index,
        "data-user-value": result.highlight ? true : undefined,
        "data-suggested-value": result.highlight ? undefined : true
      },
      str
    );
  }) : value;
}

////////////////////////////////////////////////////////////////////////////////
// The rest is all implementation details

// Move focus back to the input if we start navigating w/ the
// keyboard after focus has moved to any focusable content in
// the popup.
function useFocusManagement(lastActionType, inputRef, navigationValue) {
  (0, _react.useEffect)(function () {
    if (lastActionType === NAVIGATE || lastActionType === ESCAPE) {
      inputRef.current.focus();
    }
  });
}

// We want the same events when the input or the popup have focus (HOW COOL ARE
// HOOKS BTW?) This is probably the hairiest piece but it's not bad.
function useKeyDown() {
  var _useContext6 = (0, _react.useContext)(Context),
      navigationValue = _useContext6.data.navigationValue,
      onSelect = _useContext6.onSelect,
      optionsRef = _useContext6.optionsRef,
      state = _useContext6.state,
      transition = _useContext6.transition,
      autocompletePropRef = _useContext6.autocompletePropRef;

  return function handleKeyDown(event) {
    var options = optionsRef.current;

    switch (event.key) {
      case "ArrowDown":
        {
          // Don't scroll the page
          event.preventDefault();

          // If the developer didn't render any options, there's no point in
          // trying to navigate--but seriously what the heck? Give us some
          // options fam.
          if (!options || options.length === 0) {
            return;
          }

          if (state === IDLE) {
            // Opening a closed list, we don't want to select anything,
            // just open it and reveal the many options before us
            transition(NAVIGATE, { value: null });
          } else {
            var index = options.indexOf(navigationValue);
            var atBottom = index === options.length - 1;
            if (atBottom) {
              if (autocompletePropRef.current) {
                // Go back to the value the user has typed because we are
                // autocompleting and they need to be able to get back to what
                // they had typed w/o having to backspace out.
                transition(NAVIGATE, { value: null });
              } else {
                // cycle through
                var firstOption = options[0];
                transition(NAVIGATE, { value: firstOption });
              }
            } else {
              // Go to the next item in the list
              var nextValue = options[(index + 1) % options.length];
              transition(NAVIGATE, { value: nextValue });
            }
          }
          break;
        }
      // A lot of duplicate code with ArrowDown up next, I'm already over it.
      case "ArrowUp":
        {
          // Don't scroll the page
          event.preventDefault();

          // If the developer didn't render any options, there's no point in
          // trying to navigate--but seriously what the heck? Give us some
          // options fam.
          if (!options || options.length === 0) {
            return;
          }

          if (state === IDLE) {
            transition(NAVIGATE, { value: null });
          } else {
            var _index = options.indexOf(navigationValue);
            if (_index === 0) {
              if (autocompletePropRef.current) {
                // Go back to the value the user has typed because we are
                // autocompleting and they need to be able to get back to what
                // they had typed w/o having to backspace out.
                transition(NAVIGATE, { value: null });
              } else {
                // cycle through
                var lastOption = options[options.length - 1];
                transition(NAVIGATE, { value: lastOption });
              }
            } else if (_index === -1) {
              // displaying the user's value, so go select the last one
              var value = options.length ? options[options.length - 1] : null;
              transition(NAVIGATE, { value: value });
            } else {
              // normal case, select previous
              var _nextValue = options[(_index - 1 + options.length) % options.length];
              transition(NAVIGATE, { value: _nextValue });
            }
          }
          break;
        }
      case "Escape":
        {
          if (state !== IDLE) {
            transition(ESCAPE);
          }
          break;
        }
      case "Enter":
        {
          if (state === NAVIGATING && navigationValue !== null) {
            // don't want to submit forms
            event.preventDefault();
            onSelect && onSelect(navigationValue);
            transition(SELECT_WITH_KEYBOARD);
          }
          break;
        }
    }
  };
}

function useBlur() {
  var _useContext7 = (0, _react.useContext)(Context),
      state = _useContext7.state,
      transition = _useContext7.transition,
      popupRef = _useContext7.popupRef,
      inputRef = _useContext7.inputRef;

  return function handleBlur(event) {
    requestAnimationFrame(function () {
      // we on want to close only if focus rests outside the menu
      if (document.activeElement !== inputRef.current && popupRef.current) {
        if (popupRef.current.contains(document.activeElement)) {
          // focus landed inside the popup, keep it open, but we don't want
          // "Enter" causing the popover to close, so we clear the navigation
          // value to act like there's no navigation going on anymore (cause
          // there isn't!)
          if (state !== INTERACTING) {
            transition(INTERACT);
          }
        } else {
          // focus landed outside the popup, close it.
          transition(BLUR);
        }
      }
    });
  };
}

// This is a cool state machine, as usual, it manages transitions between
// states, but this one has a built in reducer to manage the data that goes
// with those transitions.
function useReducerMachine(chart, reducer, initialData) {
  var _useState = (0, _react.useState)(chart.initial),
      state = _useState[0],
      setState = _useState[1];

  var _useReducer = (0, _react.useReducer)(reducer, initialData),
      data = _useReducer[0],
      dispatch = _useReducer[1];

  var transition = function transition(action) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var currentState = chart.states[state];
    var nextState = currentState.on[action];
    if (!nextState) {
      throw new Error("Unknown action \"" + action + "\" for state \"" + state + "\"");
    }
    dispatch(_extends({ type: action, state: state, nextState: state }, payload));
    setState(nextState);
  };

  return [state, data, transition];
}

// We don't want to track the active descendant with indexes because nothing is
// more annoying in a combobox than having it change values RIGHT AS YOU HIT
// ENTER. That only happens if you use the index as your data, rather than
// *your data as your data*. We use this to generate a unique ID based on the
// value of each item.  This function is short, sweet, and good enough™ (I also
// don't know how it works, tbqh)
// https://stackoverflow.com/questions/6122571/simple-non-secure-hash-function-for-javascript
var makeHash = function makeHash(str) {
  var hash = 0;
  if (str.length === 0) {
    return hash;
  }
  for (var i = 0; i < str.length; i++) {
    var char = str.charCodeAt(i);
    hash = (hash << 5) - hash + char;
    hash = hash & hash;
  }
  return hash;
};

////////////////////////////////////////////////////////////////////////////////
// Well alright, you made it all the way here to like 700 lines of code (geez,
// what the heck?). Have a great day :D